@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.3/Mesh_3/include/CGAL/Mesh_3/Refine_cells_3.h $
+// $Id: Refine_cells_3.h 62a8391 2018-07-25T18:59:46+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Laurent Rineau, Stéphane Tayeb
@@ -31,8 +31,11 @@
 #include <CGAL/Mesh_3/Mesher_level_default_implementations.h>
 #include <CGAL/Meshes/Triangulation_mesher_level_traits_3.h>
 #ifdef CGAL_LINKED_WITH_TBB
-  #include <tbb/tbb.h>
+  #include <tbb/enumerable_thread_specific.h>
+  #include <tbb/blocked_range.h>
+  #include <tbb/parallel_for.h>
 #endif
+#include <CGAL/atomic.h>
 
 #include <CGAL/Meshes/Filtered_deque_container.h>
 #include <CGAL/Meshes/Filtered_multimap_container.h>
@@ -53,35 +56,6 @@ namespace CGAL {
 
 namespace Mesh_3 {
 
-// Helper meta-programming functions, to allow backward compatibility.
-//
-//   - Has_Is_cell_bad and Had_Cell_badness are used to detect if a model
-//     of the MeshCellCriteria_3 concept follows the specifications of
-//     CGAL-3.7 (with Cell_badness) or later (with Is_cell_bad).
-//
-//   - Then the meta-function Get_Is_cell_bad is used to get the actual
-//     type, either Cell_criteria::Cell_badness or
-//      Cell_criteria::Is_cell_bad.
-
-BOOST_MPL_HAS_XXX_TRAIT_NAMED_DEF(Has_Is_cell_bad, Is_cell_bad, true)
-BOOST_MPL_HAS_XXX_TRAIT_NAMED_DEF(Has_Cell_badness, Cell_badness, false)
-
-// template class, used when use_cell_badness = false
-template <typename Cell_criteria,
-          bool use_cell_badness = (!Has_Is_cell_bad<Cell_criteria>::value) &&
-                                    Has_Cell_badness<Cell_criteria>::value >
-struct Get_Is_cell_bad {
-  typedef typename Cell_criteria::Is_cell_bad Type;
-  typedef Type type; // compatibility with Boost
-};
-
-// partial specialization when use_cell_badness == true
-template <typename Cell_criteria>
-struct Get_Is_cell_bad<Cell_criteria, true> {
-  typedef typename Cell_criteria::Cell_badness Type;
-  typedef Type type;
-};
-
 // Predicate to know if a cell in a refinement queue is a zombie
 template<typename Cell_handle>
 class Cell_to_refine_is_not_zombie
@@ -301,7 +275,7 @@ private:
   typedef typename Tr::Lock_data_structure Lock_data_structure;
   typedef typename MeshDomain::Subdomain_index  Subdomain_index;
   typedef typename MeshDomain::Index  Index;
-  typedef typename Get_Is_cell_bad<Criteria>::Type Is_cell_bad;
+  typedef typename Criteria::Is_cell_bad Is_cell_bad;
 
   // Self
   typedef Refine_cells_3<Tr,
@@ -352,7 +326,11 @@ public:
                  const MeshDomain& oracle,
                  Previous_& previous,
                  C3T3& c3t3,
-                 std::size_t maximal_number_of_vertices);
+                 std::size_t maximal_number_of_vertices
+#ifndef CGAL_NO_ATOMIC
+                , CGAL::cpp11::atomic<bool>* stop_ptr
+#endif
+                );
   // For parallel
   Refine_cells_3(Tr& triangulation,
                  const Criteria& criteria,
@@ -361,7 +339,11 @@ public:
                  C3T3& c3t3,
                  Lock_data_structure *lock_ds,
                  WorksharingDataStructureType *worksharing_ds,
-                 std::size_t maximal_number_of_vertices);
+                 std::size_t maximal_number_of_vertices
+#ifndef CGAL_NO_ATOMIC
+                , CGAL::cpp11::atomic<bool>* stop_ptr
+#endif
+                );
 
   // Destructor
   virtual ~Refine_cells_3() { }
@@ -397,6 +379,13 @@ public:
   // Tells if the refinement process of cells is currently finished
   bool no_longer_element_to_refine_impl()
   {
+#ifndef CGAL_NO_ATOMIC
+    if(m_stop_ptr != 0 &&
+       m_stop_ptr->load(CGAL::cpp11::memory_order_acquire) == true)
+    {
+      return true;
+    }
+#endif
     if(m_maximal_number_of_vertices_ !=0 &&
        triangulation_ref_impl().number_of_vertices() >=
        m_maximal_number_of_vertices_)
@@ -490,10 +479,10 @@ public:
   {
     std::stringstream sstr;
     sstr << "Cell { " << std::endl
-    << "  - " << *ch->vertex(0)  << std::endl
-    << "  - " << *ch->vertex(1)  << std::endl
-    << "  - " << *ch->vertex(2)  << std::endl
-    << "  - " << *ch->vertex(3)  << std::endl
+    << "  " << *ch->vertex(0) << std::endl
+    << "  " << *ch->vertex(1) << std::endl
+    << "  " << *ch->vertex(2) << std::endl
+    << "  " << *ch->vertex(3) << std::endl
     << "}" << std::endl;
 
     return sstr.str();
@@ -545,8 +534,8 @@ private:
   // -----------------------------------
   // -----------------------------------
 
-  /// Computes badness and add to queue if needed
-  void compute_badness(const Cell_handle& cell);
+  /// Check whether the cell is bad or not, and add it to the queue if needed
+  void is_bad(const Cell_handle& cell);
 
   // Updates cells incident to vertex, and add them to queue if needed
   void update_star_self(const Vertex_handle& vertex);
@@ -573,7 +562,7 @@ private:
   }
 
   /// Get mirror facet
-  Facet mirror_facet(const Facet& f) const { return r_tr_.mirror_facet(f); };
+  Facet mirror_facet(const Facet& f) const { return r_tr_.mirror_facet(f); }
   Facet mirror_facet(const Cell_handle& c, const int i) const
   { return mirror_facet(std::make_pair(c,i)); }
 
@@ -590,6 +579,10 @@ private:
   /// Maximal allowed number of vertices
   std::size_t m_maximal_number_of_vertices_;
 
+#ifndef CGAL_NO_ATOMIC
+  /// Pointer to the atomic Boolean that can stop the process
+  CGAL::cpp11::atomic<bool>* const m_stop_ptr;
+#endif
 private:
   // Disabled copy constructor
   Refine_cells_3(const Self& src);
@@ -608,7 +601,11 @@ Refine_cells_3(Tr& triangulation,
                const MD& oracle,
                P_& previous,
                C3T3& c3t3,
-               std::size_t maximal_number_of_vertices)
+               std::size_t maximal_number_of_vertices
+#ifndef CGAL_NO_ATOMIC
+               , CGAL::cpp11::atomic<bool>* stop_ptr
+#endif
+               )
   : Mesher_level<Tr, Self, Cell_handle, P_,
       Triangulation_mesher_level_traits_3<Tr>, Ct >(previous)
   , C_()
@@ -620,6 +617,9 @@ Refine_cells_3(Tr& triangulation,
   , r_oracle_(oracle)
   , r_c3t3_(c3t3)
   , m_maximal_number_of_vertices_(maximal_number_of_vertices)
+#ifndef CGAL_NO_ATOMIC
+  , m_stop_ptr(stop_ptr)
+#endif
 {
 }
 
@@ -634,7 +634,11 @@ Refine_cells_3(Tr& triangulation,
                C3T3& c3t3,
                Lock_data_structure *lock_ds,
                WorksharingDataStructureType *worksharing_ds,
-               std::size_t maximal_number_of_vertices)
+               std::size_t maximal_number_of_vertices
+#ifndef CGAL_NO_ATOMIC
+               , CGAL::cpp11::atomic<bool>* stop_ptr
+#endif
+               )
   : Mesher_level<Tr, Self, Cell_handle, P_,
       Triangulation_mesher_level_traits_3<Tr>, Ct >(previous, lock_ds, worksharing_ds)
   , C_()
@@ -646,6 +650,9 @@ Refine_cells_3(Tr& triangulation,
   , r_oracle_(oracle)
   , r_c3t3_(c3t3)
   , m_maximal_number_of_vertices_(maximal_number_of_vertices)
+#ifndef CGAL_NO_ATOMIC
+  , m_stop_ptr(stop_ptr)
+#endif
 {
 }
 
@@ -765,7 +772,7 @@ number_of_bad_elements_impl()
     const Subdomain subdomain = r_oracle_.is_in_domain_object()(r_tr_.dual(cell_it));
     if ( subdomain )
     {
-      const Is_cell_bad is_cell_bad = r_criteria_(cell_it);
+      const Is_cell_bad is_cell_bad = r_criteria_(r_tr_, cell_it);
       if( is_cell_bad )
         ++count;
     }
@@ -915,7 +922,7 @@ update_star_self(const Vertex_handle& vertex)
     set_cell_in_domain(*cell_it, cells_subdomain);
 
     // Add to queue
-    compute_badness(*cell_it);
+    is_bad(*cell_it);
   }
 }
 
@@ -934,7 +941,7 @@ treat_new_cell(const Cell_handle& cell)
     set_cell_in_domain(cell, *subdomain);
 
     // Add to refinement queue if needed
-    compute_badness(cell);
+    is_bad(cell);
   }
   else
   {
@@ -945,9 +952,9 @@ treat_new_cell(const Cell_handle& cell)
 template<class Tr, class Cr, class MD, class C3T3_, class P_, class Ct, class C_>
 void
 Refine_cells_3<Tr,Cr,MD,C3T3_,P_,Ct,C_>::
-compute_badness(const Cell_handle& cell)
+is_bad(const Cell_handle& cell)
 {
-  const Is_cell_bad is_cell_bad = r_criteria_(cell);
+  const Is_cell_bad is_cell_bad = r_criteria_(r_tr_, cell);
   if( is_cell_bad )
   {
     this->add_bad_element(this->from_cell_to_refinement_queue_element(cell), *is_cell_bad);
