@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.3/Nef_3/include/CGAL/Nef_polyhedron_3.h $
+// $Id: Nef_polyhedron_3.h 30e2fe0 2018-03-02T10:39:57+01:00 Jane Tournois
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Michael Seel    <seel@mpi-sb.mpg.de>
@@ -25,6 +26,10 @@
 #ifndef CGAL_NEF_POLYHEDRON_3_H
 #define CGAL_NEF_POLYHEDRON_3_H
 
+#include <CGAL/license/Nef_3.h>
+
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/basic.h>
 #include <CGAL/Handle_for.h>
 #include <CGAL/Nef_3/Default_items.h>
@@ -48,20 +53,13 @@
 #include <CGAL/Modifier_base.h>
 #include <CGAL/Nef_3/Mark_bounded_volumes.h>
 
-
-#ifdef CGAL_NEF3_CGAL_NEF3_SM_VISUALIZOR
-#include <CGAL/Nef_3/SNC_SM_visualizor.h>
-#endif // CGAL_NEF3_SM_VISUALIZOR
-
-#ifdef CGAL_NEF3_OLD_VISUALIZATION 
-#include <CGAL/Nef_3/Visualizor_OpenGL_3.h>
-#endif // CGAL_NEF3_OLD_VISUALIZATION 
-
 #include <CGAL/IO/Verbose_ostream.h>
-#include <CGAL/Nef_3/polyhedron_3_to_nef_3.h>
+#include <CGAL/Nef_3/polygon_mesh_to_nef_3.h>
 #include <CGAL/Nef_3/shell_to_nef_3.h>
 #include <CGAL/Polyhedron_incremental_builder_3.h>
 #include <CGAL/Polyhedron_3.h>
+#include <CGAL/boost/graph/graph_traits_Polyhedron_3.h>
+#include <CGAL/boost/graph/properties_Polyhedron_3.h>
 #include <CGAL/Nef_3/SNC_point_locator.h>
 #include <CGAL/assertions.h>
 
@@ -73,6 +71,9 @@
 #include <CGAL/Constrained_triangulation_face_base_2.h>
 #include <list>
 
+#include <boost/type_traits/is_same.hpp>
+#include <boost/utility/enable_if.hpp>
+
 // RO: includes for "vertex cycle to Nef" constructor
 #include <CGAL/Nef_3/vertex_cycle_to_nef_3.h>
 #include <CGAL/Vector_3.h>
@@ -120,10 +121,6 @@ class Nef_polyhedron_3_rep
   typedef CGAL::SNC_SM_overlayer<I, SM_decorator>  SM_overlayer;
   typedef CGAL::SM_point_locator<SNC_structure>    SM_point_locator;
 
-#ifdef CGAL_NEF3_SM_VISUALIZOR
-  typedef CGAL::SNC_SM_visualizor<SNC_structure>       SM_visualizor;
-#endif // CGAL_NEF3_SM_VISUALIZOR
-
  private:
   SNC_structure snc_;
   SNC_point_locator* pl_;
@@ -167,8 +164,8 @@ class Nef_polyhedron_3 : public CGAL::Handle_for< Nef_polyhedron_3_rep<Kernel_,
   typedef typename Kernel::Segment_3                  Segment_3;
   typedef typename Kernel::Aff_transformation_3       Aff_transformation_3;
 
-#ifndef _MSC_VER
-  // VC++ has a problem to digest the following typedef,
+#if (! defined _MSC_VER) || (_MSC_VER >= 1900)
+  // VC++ < 2017 has a problem to digest the following typedef,
   // and does not need the using statements -- AF
   // The left and right part of these typedefs have the same name. It is
   // very important to qualify the left part with the CGAL:: namespace, no
@@ -238,12 +235,6 @@ protected:
   typedef typename Nef_rep::SM_overlayer        SM_overlayer;
   typedef typename Nef_rep::SM_point_locator    SM_point_locator;
   typedef typename Nef_rep::SNC_simplify        SNC_simplify;
-#ifdef CGAL_NEF3_SM_VISUALIZOR
-  typedef typename Nef_rep::SM_visualizor       SM_visualizor;
-#endif // CGAL_NEF3_SM_VISUALIZOR
-#ifdef CGAL_NEF3_OLD_VISUALIZATION 
-  typedef CGAL::Nef_Visualizor_OpenGL_3<Nef_polyhedron_3> Visualizor;
-#endif // CGAL_NEF3_OLD_VISUALIZATION 
 
  typedef typename Nef_rep::Sphere_map                Sphere_map;
  public:
@@ -625,6 +616,41 @@ protected:
     delegate(mbv);
     set_snc(snc());
   }
+
+ template <class PolygonMesh>
+ explicit Nef_polyhedron_3(const PolygonMesh& pm) {
+    CGAL_NEF_TRACEN("construction from PolygonMesh with internal index maps");
+    SNC_structure rsnc;
+    *this = Nef_polyhedron_3(rsnc, new SNC_point_locator_default, false);
+    initialize_infibox_vertices(EMPTY);
+    polygon_mesh_to_nef_3<PolygonMesh, SNC_structure>(const_cast<PolygonMesh&>(pm), snc());
+    build_external_structure();
+    simplify();
+    CGAL::Mark_bounded_volumes<Nef_polyhedron_3> mbv(true);
+    delegate(mbv);
+    set_snc(snc());
+  }
+
+ template <class PolygonMesh, class HalfedgeIndexMap, class FaceIndexMap>
+ explicit Nef_polyhedron_3(const PolygonMesh& pm,
+                           const HalfedgeIndexMap& him,
+                           const FaceIndexMap& fim,
+                           typename boost::disable_if <
+                              boost::is_same<FaceIndexMap, bool>
+                           >::type* = NULL // disambiguate with another constructor
+  )
+  {
+    CGAL_NEF_TRACEN("construction from PolygonMesh");
+    SNC_structure rsnc;
+    *this = Nef_polyhedron_3(rsnc, new SNC_point_locator_default, false);
+    initialize_infibox_vertices(EMPTY);
+    polygon_mesh_to_nef_3<PolygonMesh, SNC_structure>(const_cast<PolygonMesh&>(pm), snc(), fim, him);
+    build_external_structure();
+    simplify();
+    CGAL::Mark_bounded_volumes<Nef_polyhedron_3> mbv(true);
+    delegate(mbv);
+    set_snc(snc());
+  }
   
  Nef_polyhedron_3(const Nef_polyhedron& N, 
 		  SFace_const_iterator sf) 
@@ -1228,15 +1254,7 @@ protected:
   }
 
  public:
-#ifdef CGAL_NEF3_OLD_VISUALIZATION   
-  void visualize() { 
-    Visualizor sncv(*this);
-    sncv.draw();
-    //OGL::polyhedra_.back().debug();
-    OLDOGL::start_viewer();
-  }
-#endif // CGAL_NEF3_OLD_VISUALIZATION   
-   
+ 
   void clear(Content space = EMPTY)
     { *this = Nef_polyhedron_3(space); }
   /*{\Mop makes |\Mvar| the empty set if |space == EMPTY| and the
@@ -1291,6 +1309,7 @@ protected:
 		    SNC_point_locator* _pl = new SNC_point_locator_default,
 		    bool clone_pl = true,
 		    bool clone_snc = true);
+
   /*{\Xcreate makes |\Mvar| a new object.  If |cloneit==true| then the
   underlying structure of |W| is copied into |\Mvar|.}*/
   // TODO: granados: define behavior when clone=false
@@ -1399,6 +1418,10 @@ protected:
   intersection(const Nef_polyhedron_3<Kernel,Items, Mark>& N1) const
     /*{\Mop returns |\Mvar| $\cap$ |N1|. }*/ {
     CGAL_NEF_TRACEN(" intersection between nef3 "<<&*this<<" and "<<&N1);
+    if (is_empty()) return *this;
+    if (N1.is_empty()) return N1;
+    if (is_space()) return N1;
+    if (N1.is_space()) return *this;
     AND _and;
     SNC_structure rsnc;
     Nef_polyhedron_3<Kernel,Items, Mark> res(rsnc, new SNC_point_locator_default, false);
@@ -1423,6 +1446,10 @@ protected:
   join(const Nef_polyhedron_3<Kernel,Items, Mark>& N1) const
   /*{\Mop returns |\Mvar| $\cup$ |N1|. }*/ { 
     CGAL_NEF_TRACEN(" join between nef3 "<<&*this<<" and "<<&N1);
+    if (is_empty()) return N1;
+    if (N1.is_empty()) return *this;
+    if (is_space()) return *this;
+    if (N1.is_space()) return N1;
     OR _or;
     //CGAL::binop_intersection_tests_allpairs<SNC_decorator, OR> tests_impl;
     SNC_structure rsnc;
@@ -1436,6 +1463,10 @@ protected:
   difference(const Nef_polyhedron_3<Kernel,Items, Mark>& N1) const
   /*{\Mop returns |\Mvar| $-$ |N1|. }*/ { 
     CGAL_NEF_TRACEN(" difference between nef3 "<<&*this<<" and "<<&N1);
+    if (is_empty()) return *this;
+    if (N1.is_empty()) return *this;
+    if (is_space()) return N1.complement();
+    if (N1.is_space()) return Nef_polyhedron_3(EMPTY);
     DIFF _diff;
     //CGAL::binop_intersection_tests_allpairs<SNC_decorator, DIFF> tests_impl;
     SNC_structure rsnc;
@@ -1450,6 +1481,10 @@ protected:
   /*{\Mop returns the symmectric difference |\Mvar - T| $\cup$ 
           |T - \Mvar|. }*/ {
     CGAL_NEF_TRACEN(" symmetic difference between nef3 "<<&*this<<" and "<<&N1);
+    if (is_empty()) return N1;
+    if (N1.is_empty()) return *this;
+    if (is_space()) return Nef_polyhedron_3(EMPTY);
+    if (N1.is_space()) return Nef_polyhedron_3(EMPTY);
     XOR _xor;
     //CGAL::binop_intersection_tests_allpairs<SNC_decorator, XOR> tests_impl;
     SNC_structure rsnc;
@@ -1582,7 +1617,8 @@ protected:
 
    SVertex_iterator sv;
    CGAL_forall_svertices(sv, snc()) {
-     sv->out_sedge() = sv->out_sedge()->twin();
+     if (!sv->is_isolated())
+      sv->out_sedge() = sv->out_sedge()->twin();
    }
 
    SHalfedge_iterator se;
@@ -2127,4 +2163,6 @@ extract_boundary() {
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif //CGAL_NEF_POLYHEDRON_3_H
