@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/face_graph_utils.h $
+// $Id: face_graph_utils.h 3501af6 2020-02-13T14:38:19+01:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Sebastien Loriot
@@ -27,11 +28,14 @@
 #include <CGAL/Polygon_mesh_processing/orientation.h>
 #include <CGAL/property_map.h>
 #include <boost/type_traits/is_const.hpp>
+#include <boost/type_traits/is_same.hpp>
 #include <boost/type_traits/remove_reference.hpp>
+#include <boost/mpl/if.hpp>
 #include <fstream>
 #include <sstream>
 #include <set>
 namespace CGAL {
+namespace Polygon_mesh_processing {
 namespace Corefinement {
 
 template <typename G>
@@ -151,13 +155,116 @@ void copy_edge_mark(G&,
                           No_mark<G>&)
 {} // nothing to do
 
+// Parts to get default property maps for output meshes based on the value type
+// of input vertex point maps.
+template <typename Point_3, typename vertex_descriptor>
+struct Dummy_default_vertex_point_map
+{
+  typedef vertex_descriptor key_type;
+  typedef Point_3 value_type;
+  typedef Point_3 reference;
+  typedef boost::read_write_property_map_tag category;
+
+  inline friend
+  value_type
+  get(Dummy_default_vertex_point_map, key_type)
+  {
+    CGAL_assertion(false ||
+      !"This property map should not be used."
+       "Check the value type of the output vpm vs that of input");
+    return Point_3();
+  }
+
+  inline friend
+  void
+  put(Dummy_default_vertex_point_map, key_type, value_type)
+  {
+    CGAL_assertion(false ||
+      !"This property map should not be used."
+       "Check the value type of the output vpm vs that of input");
+  }
+};
+
+template <class Point_3, class NamedParameter, class PolygonMesh>
+struct TweakedGetVertexPointMap
+{
+  typedef typename GetVertexPointMap<PolygonMesh,
+                                     NamedParameter>::type Default_map;
+  typedef typename boost::is_same<Point_3,
+    typename boost::property_traits<Default_map>::value_type>::type Use_default_tag;
+
+  typedef typename boost::mpl::if_<
+    Use_default_tag,
+    Default_map,
+    Dummy_default_vertex_point_map<Point_3,
+      typename boost::graph_traits<PolygonMesh>::vertex_descriptor >
+  >::type type;
+};
+
+template <class PT, class NP, class PM>
+boost::optional< typename TweakedGetVertexPointMap<PT, NP, PM>::type >
+get_vpm(const NP& np, boost::optional<PM*> opm, boost::true_type)
+{
+  if (boost::none == opm) return boost::none;
+  return parameters::choose_parameter(
+           parameters::get_parameter(np, internal_np::vertex_point),
+           get_property_map(boost::vertex_point, *(*opm)) );
+}
+
+template <class PT, class NP, class PM>
+boost::optional< typename TweakedGetVertexPointMap<PT, NP, PM>::type >
+get_vpm(const NP&, boost::optional<PM*> opm, boost::false_type)
+{
+  if (boost::none == opm) return boost::none;
+  return typename TweakedGetVertexPointMap<PT, NP, PM>::type();
+}
+//
+
+template <class TriangleMesh>
+struct Default_visitor{
+  typedef boost::graph_traits<TriangleMesh> GT;
+  typedef typename GT::face_descriptor face_descriptor;
+
+  void before_subface_creations(face_descriptor /*f_old*/,TriangleMesh&){}
+  void after_subface_creations(TriangleMesh&){}
+  void before_subface_created(TriangleMesh&){}
+  void after_subface_created(face_descriptor /*f_new*/,TriangleMesh&){}
+  void before_face_copy(face_descriptor /*f_old*/, TriangleMesh&, TriangleMesh&){}
+  void after_face_copy(face_descriptor /*f_old*/, TriangleMesh&,
+                       face_descriptor /* f_new */, TriangleMesh&){}
+
+// calls commented in the code and probably incomplete due to the migration
+// see NODE_VISITOR_TAG
+/*
+  void new_node_added(  std::size_t node_id,
+                        Intersection_type type,
+                        halfedge_descriptor principal_edge,
+                        halfedge_descriptor additional_edge,
+                        bool is_target_coplanar,
+                        bool is_source_coplanar)
+  {}
+
+  // autorefinement only
+  void new_node_added_triple_face(std::size_t node_id,
+                                  face_descriptor f1,
+                                  face_descriptor f2,
+                                  face_descriptor f3,
+                                  const TriangleMesh& tm)
+  {}
+
+  void new_vertex_added(std::size_tnode_id,
+                        vertex_descriptor vh,
+                        TriangleMesh& tm){}
+*/
+};
+
 template < class TriangleMesh,
            class VertexPointMap,
            class Node_id,
            class Node_vector,
            class CDT,
-           class NewNodeVisitor,
-           class NewFaceVisitor  >
+           class OutputBuilder,
+           class UserVisitor>
 void
 triangulate_a_face(
   typename boost::graph_traits<TriangleMesh>::face_descriptor current_face,
@@ -171,8 +278,8 @@ triangulate_a_face(
                 ::halfedge_descriptor>& edge_to_hedge,
   const CDT& cdt,
   const VertexPointMap& vpm,
-  NewNodeVisitor& new_node_visitor,
-  NewFaceVisitor& new_face_visitor)
+  OutputBuilder& output_builder,
+  UserVisitor& user_visitor)
 {
   typedef boost::graph_traits<TriangleMesh> GT;
   typedef typename GT::vertex_descriptor vertex_descriptor;
@@ -185,8 +292,11 @@ triangulate_a_face(
   BOOST_FOREACH(Node_id node_id, node_ids)
   {
     vertex_descriptor v=add_vertex(tm);
-    new_node_visitor.new_vertex_added(node_id, v, tm);
+//    user_visitor.new_vertex_added(node_id, v, tm); // NODE_VISITOR_TAG
     nodes.call_put(vpm, v, node_id, tm);
+    // register the new vertex in the output builder
+    output_builder.set_vertex_id(v, node_id, tm);
+
     CGAL_assertion(node_id_to_vertex.size()>node_id);
     node_id_to_vertex[node_id]=v;
   }
@@ -225,7 +335,7 @@ triangulate_a_face(
   }
 
   //grab triangles.
-  new_face_visitor.before_subface_creations(current_face,tm);
+  user_visitor.before_subface_creations(current_face,tm);
   for (typename CDT::Finite_faces_iterator it=cdt.finite_faces_begin(),
                                            it_end=cdt.finite_faces_end();;)
   {
@@ -261,22 +371,22 @@ triangulate_a_face(
 
     if ( ++it!=it_end )
     {
+      user_visitor.before_subface_created(tm);
       current_face=add_face(tm);
-      new_face_visitor.before_subface_creations(current_face,tm);
+      user_visitor.after_subface_created(current_face,tm);
     }
     else
       break;
   }
+  user_visitor.after_subface_creations(tm);
 }
 
 template <class PolygonMesh>
 class Border_edge_map {
-  typedef std::size_t Node_id;
   typedef boost::graph_traits<PolygonMesh> GT;
   typedef typename GT::halfedge_descriptor halfedge_descriptor;
   typedef typename GT::edge_descriptor edge_descriptor;
-  typedef boost::unordered_map<edge_descriptor,
-                               std::pair<Node_id,Node_id> > Intersection_edge_map;
+  typedef boost::unordered_set<edge_descriptor> Intersection_edge_map;
   const Intersection_edge_map* intersection_edges;
   const PolygonMesh* tm;
 public:
@@ -483,6 +593,7 @@ template <class PolygonMesh,
           class EdgeMap,
           class VertexMap,
           class VertexPointMap,
+          class VertexPointMapOut,
           class IntersectionEdgeMap>
 void import_polyline(
   PolygonMesh& output,
@@ -498,7 +609,7 @@ void import_polyline(
   const IntersectionEdgeMap& intersection_edges2,
   const VertexPointMap& vpm1,
   const VertexPointMap& /*vpm2*/,
-  const VertexPointMap& vpm_out,
+  const VertexPointMapOut& vpm_out,
   std::vector<typename boost::graph_traits<PolygonMesh>
                 ::edge_descriptor>& output_shared_edges)
 {
@@ -695,20 +806,23 @@ template < bool reverse_patch_orientation,
            class TriangleMesh,
            class PatchContainer,
            class VertexPointMap,
+           class VertexPointMapOut,
            class EdgeMarkMapOut,
-           class EdgeMarkMapIn >
+           class EdgeMarkMapIn ,
+           class UserVisitor>
 void append_patches_to_triangle_mesh(
   TriangleMesh& output,
   const boost::dynamic_bitset<>& patches_to_append,
   PatchContainer& patches,
-  const VertexPointMap& vpm_out,
+  const VertexPointMapOut& vpm_out,
   const VertexPointMap& vpm_tm,
   EdgeMarkMapOut& edge_mark_map_out,
   const EdgeMarkMapIn& edge_mark_map_in,
   boost::unordered_map<
     typename boost::graph_traits<TriangleMesh>::edge_descriptor,
     typename boost::graph_traits<TriangleMesh>::edge_descriptor
-  >& tm_to_output_edges)
+  >& tm_to_output_edges,
+  UserVisitor& user_visitor)
 {
   typedef boost::graph_traits<TriangleMesh> GT;
   typedef typename GT::halfedge_descriptor halfedge_descriptor;
@@ -780,7 +894,9 @@ void append_patches_to_triangle_mesh(
     {
       cpp11::array<halfedge_descriptor,3> hedges = helper.halfedges(f);
 
+      user_visitor.before_face_copy(f, patches.pm, output);
       face_descriptor new_f = add_face(output);
+      user_visitor.after_face_copy(f, patches.pm, new_f, output);
       set_halfedge(new_f, hedges[0], output);
 
       for (int i=0;i<3;++i)
@@ -898,11 +1014,13 @@ void append_patches_to_triangle_mesh(
 template < class TriangleMesh,
            class IntersectionEdgeMap,
            class VertexPointMap,
+           class VertexPointMapOut,
            class EdgeMarkMap1,
            class EdgeMarkMap2,
            class EdgeMarkMapOut,
            class IntersectionPolylines,
-           class PatchContainer>
+           class PatchContainer,
+           class UserVisitor>
 void fill_new_triangle_mesh(
   TriangleMesh& output,
   const boost::dynamic_bitset<>& patches_of_tm1_to_import,
@@ -916,12 +1034,13 @@ void fill_new_triangle_mesh(
   const IntersectionEdgeMap& intersection_edges2,
   const VertexPointMap& vpm1,
   const VertexPointMap& vpm2,
-  const VertexPointMap& vpm_out,
+  const VertexPointMapOut& vpm_out,
   const EdgeMarkMap1& edge_mark_map1,
   const EdgeMarkMap2& edge_mark_map2,
         EdgeMarkMapOut& edge_mark_map_out,
   std::vector< typename boost::graph_traits<TriangleMesh>::edge_descriptor>&
-                                                            output_shared_edges)
+                                                            output_shared_edges,
+  UserVisitor& user_visitor)
 {
   typedef boost::graph_traits<TriangleMesh> GT;
   typedef typename GT::vertex_descriptor vertex_descriptor;
@@ -960,7 +1079,8 @@ void fill_new_triangle_mesh(
                                           vpm1,
                                           edge_mark_map_out,
                                           edge_mark_map1,
-                                          tm1_to_output_edges);
+                                          tm1_to_output_edges,
+                                          user_visitor);
   else
     append_patches_to_triangle_mesh<false>(output,
                                            patches_of_tm1_to_import,
@@ -969,7 +1089,8 @@ void fill_new_triangle_mesh(
                                            vpm1,
                                            edge_mark_map_out,
                                            edge_mark_map1,
-                                           tm1_to_output_edges);
+                                           tm1_to_output_edges,
+                                           user_visitor);
 
   //import patches from tm2
   if (reverse_orientation_of_patches_from_tm2)
@@ -980,7 +1101,8 @@ void fill_new_triangle_mesh(
                                           vpm2,
                                           edge_mark_map_out,
                                           edge_mark_map2,
-                                          tm2_to_output_edges);
+                                          tm2_to_output_edges,
+                                          user_visitor);
   else
     append_patches_to_triangle_mesh<false>(output,
                                            patches_of_tm2_to_import,
@@ -989,7 +1111,8 @@ void fill_new_triangle_mesh(
                                            vpm2,
                                            edge_mark_map_out,
                                            edge_mark_map2,
-                                           tm2_to_output_edges);
+                                           tm2_to_output_edges,
+                                           user_visitor);
 }
 
 template <class TriangleMesh,
@@ -1148,7 +1271,8 @@ template <class TriangleMesh,
           class VertexPointMap,
           class EdgeMarkMapIn1,
           class EdgeMarkMapIn2,
-          class EdgeMarkMapOut>
+          class EdgeMarkMapOut,
+          class UserVisitor>
 void compute_inplace_operation_delay_removal_and_insideout(
   TriangleMesh& tm1,
   TriangleMesh& tm2,
@@ -1163,7 +1287,8 @@ void compute_inplace_operation_delay_removal_and_insideout(
         EdgeMarkMapIn1&,
   const EdgeMarkMapIn2& edge_mark_map2,
   const EdgeMarkMapOut& edge_mark_map_out1,
-  EdgeMap& disconnected_patches_edge_to_tm2_edge)
+  EdgeMap& disconnected_patches_edge_to_tm2_edge,
+  UserVisitor& user_visitor)
 {
   typedef boost::graph_traits<TriangleMesh> GT;
   typedef typename GT::edge_descriptor edge_descriptor;
@@ -1213,7 +1338,8 @@ void compute_inplace_operation_delay_removal_and_insideout(
                                           vpm2,
                                           edge_mark_map_out1,
                                           edge_mark_map2,
-                                          tm2_edge_to_tm1_edge);
+                                          tm2_edge_to_tm1_edge,
+                                          user_visitor);
   else
     append_patches_to_triangle_mesh<false>(tm1,
                                            patches_of_tm2_to_import,
@@ -1222,7 +1348,8 @@ void compute_inplace_operation_delay_removal_and_insideout(
                                            vpm2,
                                            edge_mark_map_out1,
                                            edge_mark_map2,
-                                           tm2_edge_to_tm1_edge);
+                                           tm2_edge_to_tm1_edge,
+                                           user_visitor);
 }
 
 template <class TriangleMesh,
@@ -1282,7 +1409,7 @@ remove_patches(TriangleMesh& tm,
           CGAL_assertion( is_border(nxt, tm) );//we marked it above!
           // now update the prev pointer
           halfedge_descriptor prv=prev(opposite(h, tm), tm);
-          set_next(prv, next(h, tm), tm);
+          set_next(prv, nxt, tm);
           set_halfedge(target(prv, tm), prv, tm);
         }
       }
@@ -1302,7 +1429,8 @@ template <class TriangleMesh,
           class VertexPointMap,
           class EdgeMarkMapIn1,
           class EdgeMarkMapIn2,
-          class EdgeMarkMapOut1>
+          class EdgeMarkMapOut1,
+          class UserVisitor>
 void compute_inplace_operation(
         TriangleMesh& tm1,
   const TriangleMesh& /*tm2*/,
@@ -1320,7 +1448,8 @@ void compute_inplace_operation(
   boost::unordered_map<
     typename boost::graph_traits<TriangleMesh>::edge_descriptor,
     typename boost::graph_traits<TriangleMesh>::edge_descriptor
-  >& tm2_edge_to_tm1_edge)
+  >& tm2_edge_to_tm1_edge,
+  UserVisitor& user_visitor)
 {
   typedef boost::unordered_map<
       typename boost::graph_traits<TriangleMesh>::edge_descriptor,
@@ -1349,7 +1478,8 @@ void compute_inplace_operation(
                                           vpm2,
                                           edge_mark_map_out1,
                                           edge_mark_map_in2,
-                                          tm2_edge_to_tm1_edge);
+                                          tm2_edge_to_tm1_edge,
+                                          user_visitor);
   else
     append_patches_to_triangle_mesh<false>(tm1,
                                            patches_of_tm2_to_import,
@@ -1358,7 +1488,8 @@ void compute_inplace_operation(
                                            vpm2,
                                            edge_mark_map_out1,
                                            edge_mark_map_in2,
-                                           tm2_edge_to_tm1_edge);
+                                           tm2_edge_to_tm1_edge,
+                                           user_visitor);
 }
 
 template <class TriangleMesh,
@@ -1403,7 +1534,8 @@ template <class TriangleMesh,
           class VertexPointMap,
           class EdgeMarkMapIn1,
           class EdgeMarkMapIn2,
-          class EdgeMarkMapOut1>
+          class EdgeMarkMapOut1,
+          class UserVisitor>
 void compute_inplace_operation(
         TriangleMesh& tm1,
   const TriangleMesh& tm2,
@@ -1418,7 +1550,8 @@ void compute_inplace_operation(
   const EdgeMarkMapIn1& edge_mark_map_in1,
   const EdgeMarkMapIn2& edge_mark_map_in2,
   const EdgeMarkMapOut1& edge_mark_map_out1,
-  const IntersectionPolylines& polylines)
+  const IntersectionPolylines& polylines,
+        UserVisitor& user_visitor)
 {
   typedef boost::graph_traits<TriangleMesh> GT;
   typedef typename GT::edge_descriptor edge_descriptor;
@@ -1441,7 +1574,8 @@ void compute_inplace_operation(
                             edge_mark_map_in1,
                             edge_mark_map_in2,
                             edge_mark_map_out1,
-                            tm2_edge_to_tm1_edge);
+                            tm2_edge_to_tm1_edge,
+                            user_visitor);
 }
 
 // function used to remove polylines imported or kept that are incident only
@@ -1481,6 +1615,7 @@ void remove_unused_polylines(
     }
   }
 
+  std::vector<vertex_descriptor> vertices_kept;
   BOOST_FOREACH(vertex_descriptor v, vertices_to_remove)
   {
     bool to_remove=true;
@@ -1495,7 +1630,44 @@ void remove_unused_polylines(
       }
     if (to_remove)
       remove_vertex(v,tm);
+    else
+      vertices_kept.push_back(v);
   }
+
+  // update next/prev pointers around vertices in vertices_kept
+  BOOST_FOREACH(vertex_descriptor v, vertices_kept)
+  {
+    halfedge_descriptor h = halfedge(v, tm), start=GT::null_halfedge();
+
+    do{
+
+      halfedge_descriptor tmp_start = h;
+      while ( !is_border(h, tm) || is_border(opposite(h, tm), tm) )
+      {
+        h = opposite(next(h, tm), tm);
+        if (tmp_start==h) break;
+      }
+      if( !is_border(h, tm) )
+      {
+        // nothing to do: the vertex has already been updated and is now in the middle of a patch kept.
+        // This function can be called after the stitching of the patches kept, the vertex halfedge
+        // can have been updated and no border halfedge might be found
+        break;
+      }
+      halfedge_descriptor in = h;
+
+      if (start==GT::null_halfedge())
+        start=in;
+      else
+        if (start==in)
+          break;
+      while ( is_border(h, tm) )
+        h = opposite(next(h, tm), tm);
+      set_next(in, opposite(h, tm), tm);
+    }
+    while(true);//this loop handles non-manifold vertices
+  }
+
   BOOST_FOREACH(edge_descriptor e, edges_to_remove)
     remove_edge(e,tm);
 }
@@ -1538,6 +1710,6 @@ void remove_disconnected_patches(
   }
 }
 
-} } // end of namespace CGAL::Corefinement
+} } } // CGAL::Polygon_mesh_processing::Corefinement
 
 #endif // CGAL_POLYGON_MESH_PROCESSING_INTERNAL_FACE_GRAPH_UTILS_H
