@@ -12,25 +12,34 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
-// 
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/measure.h $
+// $Id: measure.h cebf065 2019-07-14T22:45:47+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
+//
 //
 // Author(s)     : Andreas Fabri
 
 #ifndef CGAL_POLYGON_MESH_PROCESSING_MEASURE_H
 #define CGAL_POLYGON_MESH_PROCESSING_MEASURE_H
 
+#include <CGAL/license/Polygon_mesh_processing/measure.h>
+
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/boost/graph/iterator.h>
 #include <CGAL/boost/graph/helpers.h>
 #include <CGAL/boost/graph/properties.h>
+#include <boost/unordered_set.hpp>
 #include <boost/graph/graph_traits.hpp>
-#include <boost/mpl/eval_if.hpp>
-#include <boost/mpl/identity.hpp>
-#include <boost/type_traits/is_base_of.hpp>
 
 #include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
 #include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
+#include <CGAL/squared_distance_3.h>
+#include <CGAL/Kernel/global_functions_3.h>
+
+#include <CGAL/Lazy.h> // needed for CGAL::exact(FT)/CGAL::exact(Lazy_exact_nt<T>)
+
+#include <utility>
 
 #ifdef DOXYGEN_RUNNING
 #define CGAL_PMP_NP_TEMPLATE_PARAMETERS NamedParameters
@@ -41,26 +50,35 @@ namespace CGAL {
 
 namespace Polygon_mesh_processing {
 
+// workaround for area(face_range,tm) overload
+template<typename CGAL_PMP_NP_TEMPLATE_PARAMETERS, typename NP>
+class GetGeomTraits<CGAL_PMP_NP_CLASS, NP>
+{
+public:
+  struct type{};
+};
+
   /**
   * \ingroup measure_grp
   * computes the length of an edge of a given polygon mesh.
   * The edge is given by one of its halfedges, or the edge itself.
   *
-  * @tparam PolygonMesh a model of `HalfedgeGraph` that has an internal property map
-  *         for `CGAL::vertex_point_t`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam PolygonMesh a model of `HalfedgeGraph`
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param h one halfedge of the edge to compute the length
   * @param pmesh the polygon mesh to which `h` belongs
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh` \cgalParamEnd
+  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
+  *   If this parameter is omitted, an internal property map for
+  *   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
   *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
   * \cgalNamedParamsEnd
   *
   * @return the length of `h`. The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `pmesh`.
   *
@@ -81,13 +99,12 @@ namespace Polygon_mesh_processing {
               , const PolygonMesh& pmesh
               , const NamedParameters& np)
   {
-    using boost::choose_const_pmap;
-    using boost::get_param;
+    using parameters::choose_parameter;
+    using parameters::get_parameter;
 
     typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type
-    vpm = choose_const_pmap(get_param(np, CGAL::vertex_point),
-                            pmesh,
-                            CGAL::vertex_point);
+    vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                       get_const_property_map(CGAL::vertex_point, pmesh));
 
     return CGAL::approximate_sqrt(CGAL::squared_distance(get(vpm, source(h, pmesh)),
                                                          get(vpm, target(h, pmesh))));
@@ -127,22 +144,23 @@ namespace Polygon_mesh_processing {
   * computes the length of the border polyline
   * that contains a given halfedge.
   *
-  * @tparam PolygonMesh a model of `HalfedgeGraph` that has an internal property map
-  *         for `CGAL::vertex_point_t`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam PolygonMesh a model of `HalfedgeGraph`
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param h a halfedge of the border polyline of which the length is computed
   * @param pmesh the polygon mesh to which `h` belongs
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh` \cgalParamEnd
-*    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
+  *   If this parameter is omitted, an internal property map for
+  *   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
+  *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
   * \cgalNamedParamsEnd
   *
   * @return the length of the sequence of border edges of `face(h, pmesh)`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `pmesh`.
   *
@@ -185,30 +203,105 @@ namespace Polygon_mesh_processing {
       CGAL::Polygon_mesh_processing::parameters::all_default());
   }
 
+  /**
+  * \ingroup measure_grp
+  * finds the longest border of a given triangulated surface and returns
+  * a halfedge that is part of this border and the length of this border.
+  *
+  * @tparam PolygonMesh a model of `HalfedgeGraph`
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  *
+  * @param pmesh the polygon mesh
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  *
+  * \cgalNamedParamsBegin
+  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
+  *   If this parameter is omitted, an internal property map for
+  *   `CGAL::vertex_point_t` must be available in `PolygonMesh`\cgalParamEnd
+  *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  * \cgalNamedParamsEnd
+  *
+  * @return a pair composed of two members:
+  *   - `first`: a halfedge on the longest border.
+  *     The return type `halfedge_descriptor` is a halfedge descriptor. It is
+  *     deduced from the graph traits corresponding to the type `PolygonMesh`.
+  *   - `second`: the length of the longest border
+  *     The return type `FT` is a number type. It is
+  *     either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+  *     or the geometric traits class deduced from the point property map
+  *     of `pmesh`
+  *
+  */
+  template<typename PolygonMesh,
+           typename NamedParameters>
+#ifdef DOXYGEN_RUNNING
+  std::pair<halfedge_descriptor, FT>
+#else
+  std::pair<typename boost::graph_traits<PolygonMesh>::halfedge_descriptor,
+            typename GetGeomTraits<PolygonMesh, NamedParameters>::type::FT>
+#endif
+  longest_border(const PolygonMesh& pmesh
+                 , const NamedParameters& np)
+  {
+    typedef typename CGAL::Kernel_traits<typename property_map_value<PolygonMesh,
+                                                                     CGAL::vertex_point_t>::type>::Kernel::FT  FT;
+    typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor halfedge_descriptor;
+
+    boost::unordered_set<halfedge_descriptor> visited;
+    halfedge_descriptor result_halfedge = boost::graph_traits<PolygonMesh>::null_halfedge();
+    FT result_len = 0;
+    BOOST_FOREACH(halfedge_descriptor h, halfedges(pmesh)){
+      if(visited.find(h)== visited.end()){
+        if(is_border(h,pmesh)){
+          FT len = 0;
+          BOOST_FOREACH(halfedge_descriptor haf, halfedges_around_face(h, pmesh)){
+            len += edge_length(haf, pmesh, np);
+            visited.insert(haf);
+          }
+          if(result_len < len){
+            result_len = len;
+            result_halfedge = h;
+          }
+        }
+      }
+    }
+    return std::make_pair(result_halfedge, result_len); 
+  }
+  
+ template<typename PolygonMesh>  
+ std::pair<typename boost::graph_traits<PolygonMesh>::halfedge_descriptor,
+           typename CGAL::Kernel_traits<typename property_map_value<PolygonMesh,
+                                                                    CGAL::vertex_point_t>::type>::Kernel::FT>
+ longest_border(const PolygonMesh& pmesh)
+ {
+   return longest_border(pmesh,
+                         CGAL::Polygon_mesh_processing::parameters::all_default());
+ }
 
   /**
   * \ingroup measure_grp
   * computes the area of a face of a given
   * triangulated surface mesh.
   *
-  * @tparam TriangleMesh a model of `HalfedgeGraph` that has an internal property map
-  *         for `CGAL::vertex_point_t`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam TriangleMesh a model of `HalfedgeGraph`
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param f the face of which the area is computed
   * @param tmesh the triangulated surface mesh to which `f` belongs
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh` \cgalParamEnd
+  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
+  *   If this parameter is omitted, an internal property map for
+  *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
   *  \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
   * \cgalNamedParamsEnd
   *
-  *@pre `f != boost::graph_traits<TriangleMesh>::%null_face()`
+  * @pre `f != boost::graph_traits<TriangleMesh>::%null_face()`
   *
   * @return the area of `f`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   *
@@ -225,15 +318,14 @@ namespace Polygon_mesh_processing {
             , const TriangleMesh& tmesh
             , const CGAL_PMP_NP_CLASS& np)
   {
-    using boost::choose_const_pmap;
-    using boost::get_param;
+    using parameters::choose_parameter;
+    using parameters::get_parameter;
 
     CGAL_precondition(boost::graph_traits<TriangleMesh>::null_face() != f);
 
     typename GetVertexPointMap<TriangleMesh, CGAL_PMP_NP_CLASS>::const_type
-    vpm = choose_const_pmap(get_param(np, CGAL::vertex_point),
-                            tmesh,
-                            CGAL::vertex_point);
+    vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                       get_const_property_map(CGAL::vertex_point, tmesh));
 
     typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor halfedge_descriptor;
     halfedge_descriptor hd = halfedge(f, tmesh);
@@ -265,22 +357,23 @@ namespace Polygon_mesh_processing {
   * @tparam FaceRange range of `boost::graph_traits<PolygonMesh>::%face_descriptor`,
           model of `Range`.
           Its iterator type is `InputIterator`.
-  * @tparam TriangleMesh a model of `HalfedgeGraph` that has an internal property map
-  *         for `CGAL::vertex_point_t`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam TriangleMesh a model of `HalfedgeGraph`
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param face_range the range of faces of which the area is computed
   * @param tmesh the triangulated surface mesh to which the faces of `face_range` belong
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh` \cgalParamEnd
+  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
+  *   If this parameter is omitted, an internal property map for
+  *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
   *  \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel` \cgalParamEnd
   * \cgalNamedParamsEnd
   *
   * @return sum of face areas of `faces`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   *
@@ -313,13 +406,7 @@ namespace Polygon_mesh_processing {
   }
 
   template<typename FaceRange, typename TriangleMesh>
-  typename CGAL::Kernel_traits<
-    typename boost::mpl::eval_if<
-      boost::is_base_of<CGAL::named_params_base, TriangleMesh>,
-      boost::mpl::identity<TriangleMesh>,
-      property_map_value<TriangleMesh, CGAL::vertex_point_t>
-    >::type
-  >::Kernel::FT
+  typename GetGeomTraits<TriangleMesh>::type::FT
   area(FaceRange face_range, const TriangleMesh& tmesh)
   {
     return area(face_range, tmesh,
@@ -330,21 +417,22 @@ namespace Polygon_mesh_processing {
   * \ingroup measure_grp
   * computes the surface area of a triangulated surface mesh.
   *
-  * @tparam TriangleMesh a model of `HalfedgeGraph` that has an internal property map
-  *         for `CGAL::vertex_point_t`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam TriangleMesh a model of `HalfedgeGraph`
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param tmesh the triangulated surface mesh
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
-  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh` \cgalParamEnd
+  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
+  *   If this parameter is omitted, an internal property map for
+  *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
   *  \cgalParamBegin{geom_traits}an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
   * \cgalNamedParamsEnd
   *
   * @return the surface area of `tmesh`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   *
@@ -380,23 +468,24 @@ namespace Polygon_mesh_processing {
   * computes the volume of the domain bounded by
   * a closed triangulated surface mesh.
   *
-  * @tparam TriangleMesh a model of `HalfedgeGraph` that has an internal property map
-  *         for `CGAL::vertex_point_t`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam TriangleMesh a model of `HalfedgeGraph`
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param tmesh the closed triangulated surface mesh bounding the volume
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * @pre `tmesh` is closed
   *
   * \cgalNamedParamsBegin
-  *  \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh` \cgalParamEnd
-  *  \cgalParamBegin{geom_traits}an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  *  \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
+  *   If this parameter is omitted, an internal property map for
+  *   `CGAL::vertex_point_t` must be available in `TriangleMesh`\cgalParamEnd
+  *  \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
   * \cgalNamedParamsEnd
   *
   * @return the volume bounded by `tmesh`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   */
@@ -407,40 +496,43 @@ namespace Polygon_mesh_processing {
 #else
   typename GetGeomTraits<TriangleMesh, CGAL_PMP_NP_CLASS>::type::FT
 #endif
-  volume(const TriangleMesh& tmesh, const CGAL_PMP_NP_CLASS& np)
+volume(const TriangleMesh& tmesh, const CGAL_PMP_NP_CLASS& np)
+{
+  CGAL_assertion(is_triangle_mesh(tmesh));
+  CGAL_assertion(is_closed(tmesh));
+
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+
+  typename GetVertexPointMap<TriangleMesh, CGAL_PMP_NP_CLASS>::const_type
+    vpm = choose_parameter(get_parameter(np, internal_np::vertex_point),
+                       get_const_property_map(CGAL::vertex_point, tmesh));
+  typename GetGeomTraits<TriangleMesh, CGAL_PMP_NP_CLASS>::type::Point_3
+    origin(0, 0, 0);
+
+  typedef typename boost::graph_traits<TriangleMesh>::face_descriptor face_descriptor;
+
+  typename GetGeomTraits<TriangleMesh, CGAL_PMP_NP_CLASS>::type::FT volume = 0.;
+  typename CGAL::Kernel_traits<typename property_map_value<TriangleMesh,
+    CGAL::vertex_point_t>::type>::Kernel::Compute_volume_3 cv3;
+
+  BOOST_FOREACH(face_descriptor f, faces(tmesh))
   {
-    CGAL_assertion(is_triangle_mesh(tmesh));
-    CGAL_assertion(is_closed(tmesh));
-
-    using boost::choose_const_pmap;
-    using boost::get_param;
-
-    typename GetVertexPointMap<TriangleMesh, CGAL_PMP_NP_CLASS>::const_type
-      vpm = choose_const_pmap(get_param(np, CGAL::vertex_point),
-                              tmesh,
-                              CGAL::vertex_point);
-    typename GetGeomTraits<TriangleMesh, CGAL_PMP_NP_CLASS>::type::Point_3
-      origin(0, 0, 0);
-
-    typedef typename boost::graph_traits<TriangleMesh>::face_descriptor face_descriptor;
-
-    typename GetGeomTraits<TriangleMesh, CGAL_PMP_NP_CLASS>::type::FT volume = 0.;
-    BOOST_FOREACH(face_descriptor f, faces(tmesh))
-    {
-      volume += CGAL::volume(origin,
-        get(vpm, target(halfedge(f, tmesh), tmesh)),
-        get(vpm, target(next(halfedge(f, tmesh), tmesh), tmesh)),
-        get(vpm, target(prev(halfedge(f, tmesh), tmesh), tmesh)));
-      exact(volume);
-    }
-    return volume;
+    volume += cv3(origin,
+      get(vpm, target(halfedge(f, tmesh), tmesh)),
+      get(vpm, target(next(halfedge(f, tmesh), tmesh), tmesh)),
+      get(vpm, target(prev(halfedge(f, tmesh), tmesh), tmesh)));
+    exact(volume);
   }
+  return volume;
+}
 
   template<typename TriangleMesh>
   typename CGAL::Kernel_traits<typename property_map_value<TriangleMesh,
     CGAL::vertex_point_t>::type>::Kernel::FT
   volume(const TriangleMesh& tmesh)
   {
+
     return volume(tmesh,
       CGAL::Polygon_mesh_processing::parameters::all_default());
   }
@@ -448,4 +540,6 @@ namespace Polygon_mesh_processing {
 }
 }
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_POLYGON_MESH_PROCESSING_MEASURE_H
