@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.3/Point_set_processing_3/include/CGAL/vcm_estimate_edges.h $
+// $Id: vcm_estimate_edges.h 2f9408f 2018-09-04T12:01:27+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s) : Jocelyn Meyron and Quentin Mérigot
 //
@@ -23,12 +24,13 @@
 
 #include <CGAL/license/Point_set_processing_3.h>
 
+#include <CGAL/disable_warnings.h>
 
 #include <CGAL/vcm_estimate_normals.h>
 
 namespace CGAL {
 
-/// \ingroup PkgPointSetProcessing
+/// \ingroup PkgPointSetProcessing3Algorithms
 /// determines if a point is on a sharp feature edge from a point set
 /// for which the Voronoi covariance Measures have been computed.
 ///
@@ -84,4 +86,6 @@ vcm_is_on_feature_edge (cpp11::array<FT,6> &cov,
 
 } // namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_VCM_ESTIMATE_EDGES_H
